function [Ei]=EI(opt,x)
//Calculates (one point) Expected improvement.
//  
// Calling Sequence
//          Ei=EI(opt,x)
//
// Parameters
//            opt: mlist of type EGO.
//            x:  matrix (mx,nx) of points at which calculates (one point) Ei, mx rows for mx data points, nx columns for each dimension.
//
// Output
//            Ei: expected improvement at points x.
//Description 
//
//          Calculates (one point) Expected improvement.
//Examples
// // 1D example
//function [y]=gpeak(x)
//
//y=sin(x*3)-exp(-(x+0.1)^2/0.01)  
//endfunction;
//
//
//ftest=gpeak;
//x_bound=[-1; 1];
// 
// 
//noise=0;
//
//X=(linspace(x_bound(1),x_bound(2),6))';// six initial points
//YExp=ftest(X); // response 
//
//Xtest=(linspace(x_bound(1),x_bound(2),100))'; // test points
//
//// creaate initial kriging model
//[kmodel] =km(X,YExp,0,'gaussian',1,0.0000001);
//kmodel.estimateNoise=0;
//
//[kmodel p]=findTheta(kmodel,0,5,'MLL','RS',100);
//
//
//// ego boundaries
//lob=[-1];
//upb=[1];
//maxit=16;
//
//opt=EGO(kmodel,lob,upb,maxit,0.00000001,'RS');
//
//x=RLHS(10,1,lob,upb);
//EI(opt,x)
//
//scf();
//plot(Xtest,EI(opt,Xtest));
//
//See also  
//              EGO
//              EGO_stop
//              EGO_best
//              EGO_tell
//              EGO_ask
//              km
//              findTheta
//              
//Authors
//          Janis Janusevskis, 3MI/EMSE
//Bibliography   
//           Jones D.R., Schonlau M. and Welch W.J. (1998), Efficient Global Optimization of Expensive Black-Box Functions, Journal of Global optimization, 13:, 455-492.
////  
//
//pause;
model=opt.kmodel;
[y v]=predict(model,x);
[mx,nx]=size(x);


s=sqrt(v);

is0=find(s==0)';// catch where s==0
s(is0)=%eps;// no to have error div by zero

u=(opt.fmin*ones(mx,1)-y)./s;

CDF=cdfnor("PQ",u,0*ones(mx,1),ones(mx,1));

PDF=1/sqrt(2*%pi)*exp(-(u.^2)/2);

Ei=s.*(u.*CDF+PDF);

if size(is0,'r')>1 then  Ei(is0)=max(zeros(is0),opt.fmin*ones(is0)-y(is0)); end; // for the case where s==0;


if ~isreal(Ei) then 
    disp(" Error: Ei is not real number!");
    pause;
   
end;

if or(Ei<0) then
    disp(" Error: Ei is negative!");
    pause;
end;


endfunction



