function [E, R2, mY, vy, Yx, vYx]=LOOCV(model,X);
//Calculates Leave One Out Cross-validation for given kriging model.
//  
//  Calling Sequence
//
//            E=LOOCV(model);
//            [E, R2]=LOOCV(model);
//            [E, R2, mY]=LOOCV(model);
//            [E, R2, mY, vy]=LOOCV(model);
//
//            [E, R2, mY, vy, Yx]=LOOCV(model,X);
//            [E, R2, mY, vy, Yx, vYx]=LOOCV(model,X);
//
//
// Parameters
//            model: kriging model, mlist of type kmodel created previously.
//            X: optional parameter. matrix of points (m*n), m - number of points, n - number of dimensions. If X is given, function will calculates Yx and vYx (predicted mean and predicted variance at every point in X for each of m kriging models, that has been constructed without point i).
//
// Output
//            E:  relative RMSE of cross-validation, scalar.
//            R2:  R square of cross-validation, scalar.
//            mY:  Kriging prediction at every design point that has not been used for model construction, column vector (m*1);
//            vy:  estimated SK variance at every design point that has not been used for model construction, column vector (m*1);
//            Yx:  if X (m*n) is given, Yx is matrix (m,m), where Yx(j,i) is predicted mean for point X(j,:) using kriging model with i-th point not used for model construction. %nan otherwise.
//            vYx: if X (m*n) is given, vYx is matrix (m,m), where vYx(j,i) is estimated SK variance mean for point X(j,:) using kriging model with i-th point not used for model construction.%nan otherwise.
//
//
//Description
//
//Calculates Leave One Out Cross-validation for given kriging model.
//It uses estimated model.theta, Beta_kriging, model.sigma, model.noise and model.coefcov and does not reestimate these parameters.
//
// Note:
//          This function uses global variables T_kriging R_kriging Beta_kriging Y_kriging S_kriging F_kriging.
//
// Comment:  Calculates the inverse of R_kriging.
//
//
//Examples
//// 2D example
//// unknown parameters
//
//function [f]=cmb_2(x)
// x1=x(:,1);x2=x(:,2);
//f=(4-2.1*x1.*x1+(x1^4)/3).*x1.*x1+x1.*x2+(-4+4*x2.*x2).*x2.*x2
//endfunction
//
//
//x_bound=[-1 -1; 1 1];//for 2- hump Camel back function
// 
//ftest=cmb_2;
//X=RLHS(10,2,x_bound(1,:),x_bound(2,:));
//YExp=ftest(X);
//
//noise=0;theta=[1 1];
//[kmodel] =km(X,YExp,0,'gaussian',theta,noise);// creates kriging model
//  
//lob   = [0.1]*ones(1,2);upb   = [10]*ones(1,2);
//[kmodel p]=findTheta(kmodel,lob,upb,'MLL','RS',200);// searches for optimal thetas, sigma and noise
//[E,R]=LOOCV(kmodel);
//printf("Rel. RMSE of Cross Validation: %f;  Rsquare: %f;\n",E,R);
//
//
//
//See also
//           km
//           logLL
//           findTheta
//           simGaussian
//           SimCondGaussian
//Authors
//          Janis Janusevskis, 3MI/EMSE
//Bibliography 
//
//
//
  
global T_kriging R_kriging Beta_kriging Y_kriging S_kriging F_kriging;

my=zeros(model.m,1);
vy=zeros(model.m,1);

if ~exists('X','local') then X=%nan; end;
Yx=%nan;
vYx=%nan;

if X~= %nan then // if X was given
Yx=zeros(model.m,model.m);
vYx=zeros(model.m,model.m);
end;


I=eye(model.m,model.m);
iT=T_kriging\((T_kriging')\I);// calculates the inverse of R.
	//iT=inv(T_kriging); // check this
	//iT=iT*iT';//
I=[I(:,$) I(:,1:$-1)]; // transformation matrix.
//cvR=R_kriging;


for i=1:model.m  /// leaves out i-th point
    
  I=eye(model.m,model.m);
  I=[I(:,1:i-1) I(:,$) I(:,i:$-1)];  
  invT=I*iT*I';// sends i-th. row and column to m-th.
  //cvR=I*R_kriging*I'; 
  
  //P=cvR(1:i-1,1:i-1);
  
  //P1=invT(1:model.m-1,1:model.m-1);
  //Q1=invT(1:model.m-1,model.m);
  //R1=invT(model.m,1:model.m-1);
  //S1=invT(model.m,model.m);

  // gets inv of P inverse without last row and column uses Woodbury, Sherman & Morrison matrix identities, to obtain:
  //  invP=P1-Q1*inv(S1)*R1;
  invP=invT(1:model.m-1,1:model.m-1)-invT(1:model.m-1,model.m)*inv(invT(model.m,model.m))*invT(model.m,1:model.m-1);
  
  x=S_kriging(i,:);
   
  f=(model.trend(x))';
  kx=model.sigma*CorrVectorData(model,x);// covariance between data point end experiments
  kx=[kx(1:i-1); kx(i+1:$)];
  kxinvP=kx'*invP;
  my(i)=f'*Beta_kriging+(1/model.coefcov)*(kxinvP*([Y_kriging(1:i-1); Y_kriging(i+1:$)]-[F_kriging(1:i-1,:); F_kriging(i+1:$,:)]*Beta_kriging));// mean
  vy(i)=model.sigma+model.noise - (1/model.coefcov)*(kxinvP*kx);// + model.coefcov*tmp*frf*tmp'; // variance of SK


  if ~isnan(X)  then // if X was given
      for j=1:size(X,'r')
      f=(model.trend(X(j,:)))';
      kx=model.sigma*CorrVectorData(model,X(j,:));// covariance between data point end experiments
      kx=[kx(1:i-1); kx(i+1:$)];
      kxinvP=kx'*invP;
      Yx(j,i)=f'*Beta_kriging+(1/model.coefcov)*(kxinvP*([Y_kriging(1:i-1); Y_kriging(i+1:$)]-[F_kriging(1:i-1,:); F_kriging(i+1:$,:)]*Beta_kriging));
      vYx(j,i)=model.sigma+model.noise - (1/model.coefcov)*(kxinvP*kx); // variance of SK
      
     end;
  end;//if
end;//for

  mY=my;
  E=sum((Y_kriging-my).^2);
  
    // callculates R_kriging square 
  R2=1-E/sum((Y_kriging-mean(Y_kriging)).^2);
  
  // calculates RRMSE
  MSE=(E)/model.m;
  vY=variance(Y_kriging); if vY==0 then vY=1;end;
  E=100*sqrt(MSE/vY);
  

  
  
endfunction;

