function [Sim]=SimCondGaussian (Xnew,model,ns)
//Creates  conditional gaussian process simulations.
//
//Calling Sequence
//
//            Sim=SimCondGaussian (Xnew,model,ns);
//            
// Parameters
// Input
//            Xnew: data points for simulations (mx*n). Row vectors representing data points.
//            model: kriging model, mlist of type kmodel created previously.
//            ns: number of simulations. Default value ns=1.
//
// Output
//           Sim: matrix containing simulated function values at data points Xnew (m*ns).
//Description
//          
//Creates  Conditional Gaussian process simulations at Xnew data points, given observations and covariance model by kriging model.
// Note-
//          This function uses gobal variable S_kriging, T_kriging.
//Examples
//// 1D example
//x_bound=[-2; 2];
//X=(linspace(x_bound(1),x_bound(2),4))'; // Points of experiment
//YExp=sin(X);// Response values
//noise=0.0000001;theta=1;
//[kmodel] =km(X,YExp,0,'gaussian',theta,noise);// creates kriging model  (constant trend)
//Xtest=(linspace(x_bound(1),x_bound(2),100))'; // 100 point test sample
//[Sim1]=SimCondGaussian(Xtest,kmodel,50); 
//scf;
//plot(Xtest,Sim1,'k'); // simulations
//See also
//           km
//           findTheta
//           simGaussian
//Authors
//          Janis Janusevskis, 3MI/EMSE
//Bibliography 
//
//


if ~exists('ns','local')| ns<1 then ns=1;end;

//Sim=E(Ynew|Yold)+Kcond^(1/2)*randomMultivariateGaussian.
//Yold - Multivariate Gaussian on data points used for model construction, covariance represented by kriging model. 
//Ynew - Multivariate Gaussian on Xnew data points, covariance represented by kriging model.
//E(Ynew|Yold) - conditional expectation, values given by kriging prediction.
//Kcond - conditional covariance matrix. Kcond= K11-K12*inv(K22)*K21, where K11 covariance between values at Xnew, K22 covariance between values at Xold, K12=K21' covarianve between values at Knew and Xold, 


global S_kriging;

[m,n]=size(Xnew);

// scaling of data points
// no scaling actualy implemented see km.sci
//X = (X- (ones(nx,1) .*. model.scS(1,:))) ./ (ones(nx,1) .*. model.scS(2,:));
Xnew = (Xnew- (ones(m,n) .*. model.scS(1,:))) ./ (ones(m,n) .*. model.scS(2,:));

n=size(Xnew,'r');

X=[Xnew; S_kriging];
nx=size(X,'r');
m=size(S_kriging,'r');

// creates covariance matrix
P=createCorr(X,model.theta,model.cov);
P=model.sigma*P+eye(P)*model.noise;

// splits into sub-matrices
K11=P(1:n,1:n);
K22=P(n+1:n+m,n+1:n+m);
K21=P(n+1:n+m,1:n);

global T_kriging;
T1=T_kriging*sqrt(model.coefcov);
Tk21=(T1')\K21;

// conditionl covariance
Knew=K11-(Tk21')*Tk21;

U=chol(Knew);

// mean at new data points
Ynew=predict(model,Xnew);

// scales predicted values
Ynew = (Ynew - (ones(n,1) .*. model.scY(1,:))) ./ (ones(n,1) .*. model.scY(2,:));

rand('normal');
for i=1:ns 
  // creats conditional simulation
  Sim(:,i)=Ynew+U'*grand(1,'mn',zeros(n,1),eye(n,n));//rand(n,1);
  
  //scales back simulated values
  Sim(:,i) = (ones(n,1) .*.model.scY(1,:) + (ones(n,1) .*.model.scY(2,:) .* Sim(:,i)));
end;


endfunction; 


