
function [kmodel, perf]=findTheta(model,lob,upb,crit,method,maxiter,params)
//Estimates kriging hyper-parameters (corelation scale parameters (theta) and/or process variance (sigma) or proporionality coefficent between process variance and noise (alfa)), by minimizing negative Likelihood or minimizing Leave One Out Cross validation.
//  
// Calling Sequence 
//              [kmodel, perf]=findTheta(model,lob,upb,crit,method,maxiter,params);
//
// Parameters
//            model: mlist of type kmodel created previously using function km.
//            lob: row vector (1*model.n). Lower boundaries for optimization over theta.
//            upb: row vector (1*model.n). Upper boundaries for optimization over theta.
//            crit: criterion for optimization.
//            <parls>  if =='MLL' optimization uses Likelihood;
//            <parls> if == 'CV' optimization uses Leave-One-Out Cross-validation;
//            method: optimization method;
//            <parls>  if == 'CMA' optimization is based on CMA-ES.
//            <parls>  if == 'NM' optimization is based on Nelder-Mead method with restarts.
//            <parls>  if == 'RS' optimization is based on random search.
//            <parls> if == 'RLHS' optimization is based on random Latin Hypercubes.
//            maxiter: maximum number of criterion evaluations.
//            params: additional parameters for optimizer; If not given maxiter for number of maximum objective evaluations and for other default parameters are used.
//													<parls>  if method == 'CMA', params may specify CMA-ES input structure. 
//													<parls>  if method == 'NM', params may specify parameters for Nelder-Mead method with restarts.
//            kmodel: mlist of type kmodel, that contains updated kriging model with optimal parameters.
//            perf:  mlist structure of type perf containing data of performance of optimization.
//            <parls>  iter - total number of criterion evaluations.
//            <parls>  imprIter - number of iterations, where criterion value was improved.
//            <parls> models  -  list of kmodel objects for each iteration.
//            <parls>  imprModels - list of kmodel objects for iterations, where criterion value was improved.
//            <parls>  crit    -   column vector (perf.iter*1) containing criterion values for each iteration.
//            <parls>  imprCrit  -  column vector (perf.imprIter*1) containing criterion values for iterations, where criterion value was improved.
//           Fields models, imprModels,crit and imprCrit are stored only if the output parameter perf is passed in function call. 
//
//
//  Description
//Optimizes over correlation function's hyper-parameters of previously created kriging model by km(.). Determines corelation scale parameters (theta) and/or process variance (sigma) or proporionality coefficent between process variance and noise (alfa), by minimizing negative Likelihood or minimizing LOO Cross validation.
//          
//          This function sets and uses global variables R_kriging T_kriging Ty_kriging Tyfb_kriging M_kriging Y_kriging  frf_kriging Beta_kriging.
//
//Examples
// //See demos directory for more examples.
//See also
//           km
//           predict
//           LL_driver_optim
//           simGaussian
//           SimCondGaussian
//
//Authors
//          Janis Janusevskis, 3MI/EMSE
//Bibliography 
//
//

global R_kriging dR_kriging T_kriging Ty_kriging Tyfb_kriging M_kriging Y_kriging  frf_kriging Beta_kriging; 


if ~exists('model','local')| typeof(model)~='kmodel' then error('Initial kriging model should be given.');end;
if ~exists('lob','local') | ~exists('upb','local')  then error('Lower and upper boundaries on theta should be specified.');end;
[tm,tn]=size(lob);
if tm~=1 | tn ~= model.n then  error('lob should be row vector of size 1*n.'); end;
[tm,tn]=size(upb);
if tm~=1 | tn ~= model.n then  error('upb should be row vector of size 1*n.'); end;
if ~exists('crit','local')| (crit~='MLL' & crit~='CV') then error('crit should be MLL or CV.');end;
if ~exists('method','local')|(method~='CMA' & method~='NM' & method~='RS' & method~='RLHS' & method~='grid') then error('method should be specified.');end;
if ~exists('maxiter','local') then disp('Warrning: maxiter not given assumes maxiter=200');maxiter=200; end;
if ~exists('params','local') then params=%nan; end;


perf=mlist(['perf','iter','imprIter','models','imprModels','crit','imprCrit'],0,0,list(),list(),[],[]); //performance structure
 
[nargout,nargin] = argn();// if nargout >1 then saves models and criterions in perf structure
 
  
  //optimization parameters, bounds, initial point
  
  
      if model.estimateNoise==-1 then 
        //X=[thetas;alfa];
        lob=[lob 0]; 
        upb=[upb 1-%eps*10];
	  
			    // initial point
        //if model.alfa ~= %nan then 
							if ~isnan(model.alfa) then 
															x0=[model.theta  model.alfa];
													else
															x0=[model.theta  0.5]; // all variance explained by sigma %eps for numerical stability...
								end;
								//pause;
	  end;//model.estimateNoise


	  if model.estimateNoise==0 then 
	  //just thetas

	  // initial data point
	  x0=[model.theta];
	  end;//model.estimateNoise


	  if model.estimateNoise==1 & model.estimateSigma ==-1 then 
	  //X=[thetas;Sigma]
	  lob=[lob 0]; 
	  upb=[upb 10*variance(Y_kriging)];
	  //n=n+1;


								// initial data point
								//if model.sigma ~= %nan then 
								if ~isnan(model.sigma) then 
													x0=[model.theta  model.sigma];
													else
													x0=[model.theta  1];// initial sigma=1
													end;
					
					end;//model.estimateNoise



if method =='CMA' then // ask and tell optimizer. CMAES::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

	 a=[];
restarts=%t;

	  // creates  CMA  optimizer
			if ~isnan(params) then p=params;
			else
				p = struct();
				p.x0 = x0';  // mandatory: initial solution vector  
				p.sigma0 = 0.3;     // mandatory: initial standard deviation on x0 
				p.opt.scaling_of_variables = (upb -lob)'; 
				p.stop.maxfunevals= maxiter;
				p.verb.plotmodulo=0; // no ploting
				if model.silent then p.verb.displaymodulo=0;p.verb.logmodulo=0;end; // no output
				p.stop.tolfun =model.MLL*10^-4; // stop optimization if improvement in objective is less that this
				p.stop.readme.tolx =0.01;// 'stop if x-changes are smaller than tolx*sigma0*scaling';  
			//	p.opt.lambda=4+floor(3*log(N));
				a=[];
				
			end;

      					 
	 if crit=='MLL' then 
			bestValue=model.MLL;
     elseif crit=='CV' then
          bestValue=LOOCV(model);
     end;
    
	// save initial data
      bestModel=model;
        
     //save global variables
      bestT=T_kriging;  bestM=M_kriging; bestR=R_kriging;  bestdR=dR_kriging; 
      bestTy=Ty_kriging; bestTyfb=Tyfb_kriging;
      bestfrf=frf_kriging;bestBeta=Beta_kriging;
      violatedConstriants=0;
      
	  
while (perf.iter<maxiter)&(restarts)
	  if perf.iter>1 then 
		p.x0=rand(lob','uniform').*(upb'-lob')+lob';
		end;//if restarts we start from new location each time
	  
	  opt= cma_new(p);  // get an cma "object", constructor call
      
	  // starts cma ask & tell
      while ~cma_stop(opt) 
            
          X=cma_ask(opt);
                        
          for i = 1:length(X)
          
            
            // check if any of the parameters is out of bounds.
             k=0;             
             while or(X(i)+a<lob')& or(X(i)+a>upb') & k <100000000000
                  X(i) = cma_ask(opt, 1); 
                  k=k+1;
            end;
            
            // if X still violates constraints, sets to border values
            for j=1:size(X(i),'r')
                if X(i)(j)+a<lob(j) then
                     // disp(Par,'Warning: X violates lower constraints:'); 
                      //disp(it,'Iteration:'); 
                       violatedConstriants=violatedConstriants+1;
                      //Par(j)=lob(j);
                      X(i)(j)=lob(j)-a;
                      
                end;
              
                if X(i)(j)+a>upb(j) then
                      violatedConstriants=violatedConstriants+1;
                      //disp(Par,'Warning: X violates upper constraints:'); 
                      //disp(it,'Iteration:'); 
                      //Par(j)=upb(j);
                      X(i)(j)=upb(j)-a;
                      
                end;
              
            end;//for              
            Par=X(i)+a; 
            
            
            // parses optimization parameters to kriging model parameters
            if model.estimateNoise==-1 then
            model.alfa=Par($);
            model.theta=Par(1:$-1)';
            elseif model.estimateNoise==1 & model.estimateSigma ==-1 then
            model.sigma=Par($);
            model.theta=Par(1:$-1)';
            else
                 model.theta=Par(:)';
                // if model.n==1& size(model.theta,'c')==2 then  model.theta=model.theta(1); end;
            end;
       
							// creates kriging model
       
          model=CorrMatrix(model);
        
         decompose_estBeta(model);
         [model]=EstimateVar(model);
       
							// get criterion value
							 if crit=='MLL' then 
          [model]=logLL(model);
          y(i)=model.MLL;
        elseif crit=='CV' then
          y(i)=LOOCV(model);
         end;
          
          
          
        if y(i)<bestValue then 
        //saves model data for best case

          bestValue=y(i);
          bestModel=model; 
                      
           bestT=T_kriging;  bestM=M_kriging; bestR=R_kriging;  bestdR=dR_kriging; 
           bestTy=Ty_kriging; bestTyfb=Tyfb_kriging;
           bestfrf=frf_kriging;bestBeta=Beta_kriging;
            
            //performance data
            perf.imprIter=perf.imprIter+1;
            
            if nargout >1 then
                perf.imprModels($+1)=model;
                perf.imprCrit($+1)=y(i);
            end;//if
      end;
      //pause;
       
        //performance data
        perf.iter=perf.iter+1;
          if nargout >1 then
                perf.models($+1)=model;
                perf.crit($+1)=y(i);
            end;//if
      
      end;// end for i
         // pause
          opt=cma_tell(opt,X,y');
       
       

		end;// while cma_stop
		
	if ~isnan(params) then restarts=%f;end; // if optimizer parameters specified do not restart	
 end;//while restarts

if ~model.silent then         printf("Constraints on X violated %d times \n",violatedConstriants); end;
	
//         printf("Best optimizer value %d. Parameters %d\n",best(opt));
// pause;
//=========================================================================================================================
elseif (method =='NM') then //  Nelder Mead with Latin Hypercube restarts ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

global model_kriging; // needed for LL_driver_optim
	
	bestValue=%inf;// 
	  

			if ~isnan(params) then
										
										
										X=params.X; //restart points

										NmbrRestart=size(X,'r');
						// defines simplex search 
										ItMX            = params.ItMX; // maximum number of descent steps
										TOL             = params.TOL; // accuracy for convergence test - derivatives. 
																																		// The tolerance is tested on the absolute variation of objective function found on the simplex.
																																		// if abs(f_Good - f_Worth)<TOL then stop
										Log             = params.Log;   // print info from line step, 1 = on / 0 = off
										MaxEvalFunc     = params.MaxEvalFunc;
										KelleyRestart   = params.KelleyRestart;
										KelleyAlpha     = params.KelleyAlpha;
										SimplexRelSize  = params.SimplexRelSize;
   
			else
										NmbrRestart=floor(0.002*maxiter)+1; // number of restart points

						// defines simplex search 
										ItMX            = ceil((maxiter-NmbrRestart)/NmbrRestart);  // maximum number of descent steps
										TOL             = 1e-8; // accuracy for convergence test - derivatives. 
																																		// The tolerance is tested on the absolute variation of objective function found on the simplex.
																																		// if abs(f_Good - f_Worth)<TOL then stop
										Log             = %f;   // print info from line step, 1 = on / 0 = off
										MaxEvalFunc     = ceil((maxiter-NmbrRestart)/NmbrRestart);
										KelleyRestart   = %F;
										KelleyAlpha     = 1e-7;
										SimplexRelSize  = 0.1;
					
										nx=size(lob,'c');
										X=RLHS(NmbrRestart-1,nx,lob,upb); //restart points
     end;   

    
					X=[X;x0];
					
					// global a;						a=0;
					

					// save initial data
					bestModel=model;
     //save global variables
      bestT=T_kriging;  bestM=M_kriging; bestR=R_kriging;  bestdR=dR_kriging; 
      bestTy=Ty_kriging; bestTyfb=Tyfb_kriging;
      bestfrf=frf_kriging;bestBeta=Beta_kriging;


		for j=1:NmbrRestart  do
										x0=X(j,:);
					
										// We set the initial simplex
										x_init=[];
										for i=1:size(x0,'c')+1 do
										x_init(:,i) = x0' + SimplexRelSize*0.5*((upb' - lob').*grand(x0','unf',0,1) + lob');
										end;

										model_kriging=model;

										[x_opt, x_hist] = optim_nelder_mead(Costfunction, x_init, ItMX, TOL, MaxEvalFunc, Log, KelleyRestart, KelleyAlpha);
						
						if or(x_opt' <lob) then x_opt(find(x_opt'<lob))=lob(find(x_opt'<lob));	end;
						if or(x_opt' >upb) then x_opt(find(x_opt'>upb))=upb(find(x_opt'>upb));end;


            // parses optimization parameters to kriging model parameters
            if model.estimateNoise==-1 then
            model.alfa=x_opt($);
            model.theta=x_opt(1:$-1)';
            elseif model.estimateNoise==1 & model.estimateSigma ==-1 then
            model.sigma=x_opt($);
            model.theta=x_opt(1:$-1)';
            else
                 model.theta=x_opt(:)';
                // if model.n==1& size(model.theta,'c')==2 then  model.theta=model.theta(1); end;
            end;
       

							// creates kriging model
          model=CorrMatrix(model);
         decompose_estBeta(model);
         [model]=EstimateVar(model);
       
							// get criterion value
							 if crit=='MLL' then 
          [model]=logLL(model);
          y=model.MLL;
        elseif crit=='CV' then
          y=LOOCV(model);
					   end;
          
        if y<bestValue then 
        //saves model data for best case

          bestValue=y;
          bestModel=model; 
                      
           bestT=T_kriging;  bestM=M_kriging; bestR=R_kriging;  bestdR=dR_kriging; 
           bestTy=Ty_kriging; bestTyfb=Tyfb_kriging;
           bestfrf=frf_kriging;bestBeta=Beta_kriging;
            
            //performance data
            perf.imprIter=perf.imprIter+1;
            
            if nargout >1 then
                perf.imprModels($+1)=model;
                perf.imprCrit($+1)=y;
            end;//if
      end;
      //pause;
       
        //performance data
        perf.iter=perf.iter+size(x_hist);
          if nargout >1 then
                perf.models($+1)=model;
                perf.crit($+1)=y;
            end;//if
      
					end;// for iter

///////////////////////////////: optimization without ask and tell -  RS,RLHS and grid::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
else  
    //  bestValue=model.MLL;
      bestValue=%inf;// needed because MLL is different for estimation of different parameters
      
     
      // creates  sampling set for theta based on method
						nx=size(lob,'c');

    if method== 'RS' then
      Par=grand(maxiter,nx,'def').*(ones(maxiter,1).*.(upb-lob))+ones(maxiter,1).*.(lob);
    
    elseif method =='RLHS' then
      Par=RLHS(maxiter,nx,lob,upb); 
    elseif method =='grid' then 
										nmbr=floor(sqrt(maxiter));
										for ii=1:nx 
															tmp(:,ii)=(linspace(lob(ii),upb(ii),nmbr))';
										end;
										Par=[];
										for ii=1:nmbr
															for jj=1:nmbr 
													  Par=[Par; tmp(ii,1) tmp(jj,2)];
													  end;
								  end;
										maxiter= nmbr*nmbr;
					//pause;
    else 
      // error
    end;    // save initial data
      bestModel=model;
      
      
     //save global variables, not needed if global not used 
      bestT=T_kriging;  bestM=M_kriging; bestR=R_kriging;  bestdR=dR_kriging; 
      bestTy=Ty_kriging; bestTyfb=Tyfb_kriging;
      bestfrf=frf_kriging;bestBeta=Beta_kriging;
    
     
      
      
      for i=1:maxiter 

         if model.estimateNoise==-1 then
            model.alfa=Par(i,$);
            model.theta=Par(i,1:$-1);
          elseif model.estimateNoise==1 & model.estimateSigma ==-1 then

            model.sigma=Par(i,$);
            model.theta=Par(i,1:$-1);
         else
      
            model.theta=Par(i,:);
         end;
 
         model=CorrMatrix(model);
        
        //[T,Tt,Ty]=decompose_estBeta(R,model.Y);
         decompose_estBeta(model);
         [model]=EstimateVar(model);
     
       
       if crit=='MLL' then 
        [model]=logLL(model);
        y=model.MLL;
        elseif crit=='CV' then
        y=LOOCV(model);
     end;
     
         if y<bestValue then 
         
          bestValue=y;
         //saves model data for best case
          bestModel=model; 
                      
           bestT=T_kriging;  bestM=M_kriging; bestR=R_kriging;  bestdR=dR_kriging; 
           bestTy=Ty_kriging; bestTyfb=Tyfb_kriging;
           bestfrf=frf_kriging;bestBeta=Beta_kriging;
            
            //performance data      
            perf.imprIter=perf.imprIter+1;
              if nargout >1 then
                perf.imprModels($+1)=model;
                perf.imprCrit($+1)=y

            end;//if
       end;
     
        //performance data
        perf.iter=perf.iter+1;
          if nargout >1 then
                perf.models($+1)=model;
                perf.crit($+1)=y

          end;//if
          

  end;//for maxiter
  
end;//if method

  // sets variables back for best case
      model=bestModel;
  
      T_kriging=bestT;  M_kriging=bestM; R_kriging=bestR;  dR_kriging=bestdR; 
      Ty_kriging=bestTy; Tyfb_kriging=bestTyfb;
      frf_kriging=bestfrf;Beta_kriging=bestBeta;
    
      
 if ~model.silent then     
	printf(" Parameter optimization completed.");
	printf(" Method: "+string(method));
	printf("; Iterations: "+string(perf.iter));
	printf("; Improved Iterations: "+string(perf.imprIter));
	printf("; Criterion: "+string(crit));
	printf("; Best value: "+string(bestValue)+"\n");
	str=string(model.theta);strT="";
	for i=1:size(str,'c') strT=strT+ "  "+str(:,i);end;
	printf(" Thetas: "+strT+'\n');

	if model.estimateNoise==-1 then
	  printf(" Alfa: "+string(model.alfa)+'\n');
	elseif model.estimateNoise==1 & model.estimateSigma ==-1 then
	  printf(" Sigma: "+string(model.sigma)+'\n');
	end;
end;


model.optimizer = mlist(['optimizer','lob','upb','crit','method','maxiter','params'],lob,upb,crit,method,maxiter,params);
	
kmodel=model;
endfunction

function [ f  ] = Costfunction( x )
//cost functions for Simplex search
// maps x to transpose of x
// penelizes if x out of bounds
//
	xp=x';
	if or(xp<lob) then
				
					xp(find(xp<lob))=lob(find(xp<lob));
end;

if or(xp>upb) then
					xp(find(xp>upb))=upb(find(xp>upb));
end;

//xp is the x projection on the bounds for out of bounds components, otherwise it is x

					f = LL_driver_optim ( xp );

//penelize function
					if abs(f) >10^-8 then
										f=f+abs(f)*norm(xp-x')/norm(upb-lob);
					else
					// if funct(xp) is almost zero, penalize using function value at the center of domain 
										f=f+abs(model.MLL*10*(((upb-lob)/2+lob)))*norm(xp-x')/norm(upb-lob);
					
					// if it is still almost zero just use 1
										if abs(f) <10^-8 then 
															f=f+1*norm(xp-x')/norm(upb-lob);
										end;

						end;
endfunction

