function  [kmodel] = km(Exp, Resp, trend, fcov, theta, inNoise,inBeta,inSigma,silent)  
// Creates kriging model, based on given parameters. It may estimate some parameters analytically.
//
//
//Calling Sequence 
//            [kmodel]=km(Exp, Resp, trend, fcov, theta, inNoise,inBeta,inSigma,silent);
//            [kmodel]=km(Exp, Resp, trend, fcov, theta, inNoise,inBeta,inSigma);
//            [kmodel]=km(Exp, Resp, trend, fcov, theta, inNoise,inBeta);
//            [kmodel]=km(Exp, Resp, trend, fcov, theta, inNoise);
//            [kmodel]=km(Exp, Resp, trend, fcov, theta);
//
// Parameters
//            Exp: design of experiments (m*n matrix), m - number of points, n - number of dimensions (each row represents one point).
//            Resp: column vector (m*1) of m respone values for experiment.
//            trend: pointer to trend function, that should return 1*p trend coefficients; Trend parameter can also have scalar values for included trend functions:
//            <parls>  if trend ==0 then uses constant trend (function poly0);
//            <parls>  if trend==1 linear polynomial (function poly1); if trend ==2 quadratic polynomial (function poly2).
//           <parls>   if trend is not specified assumes trend=0.
//            fcov:  string describing correlation function. Use fcov= 'gaussian', Gaussian correlation; fcov= 'matern_5_2' for Matern 5_2 correlation function; fcov= 'exponential_1' or fcov='exponential_1_9' for power exponential. 
//            theta: row array (1*n) of scaling hyper-parameter values.
//            inNoise: scalar, variance of noise effect.
//            <parls>  if inNoise ==0, assumes no noise (sets noise=%eps*10 for numerical stability). If needed larger noise for numerical stability, but estimation procedure of other parameters as for deterministic case, one can use inNoise>0 in km(.)  and set manually model.estimateNoise=0 between call to km(.) and findTheta(.).
//            <parls>  if inNoise is not given, tries to estimate noise;
//            <parls>  if inNoise is given assumes non zero noise.
//            inBeta:  column vector (p*1) of trend coefficients. If zero trend needed use trend=0 and inBeta=0; 
//            <parls>  If inBeta is not given, estimates trend coefficients (Beta) analytically from the data using maximum Likelihood. In this case the estimation error is acounted in predicted variance. 
//            inSigma: scalar, total process variance. 
//            <parls> If inSigma is not given, tries to estimate  total process variance from the data using maximum likelihood.
//            silent:   if it is necessary to operate in silent mode (no output to console). Default %f - the output is displayed;
//
//            kmodel: mlist of type kmodel, containing information of kriging model.
//            <parls>  m - number of experiment points.
//            <parls>  n -  number of dimensions.
//            <parls>  trend - handle to trend funciton.
//            <parls>   cov - correlation function.
//            <parls>  theta - row array (1*n) of scaling hyper-parameter values.
//            <parls>  MLL - negative Likelihood value.
//            <parls>  dMLL - derivative of likelihood. Not implemented.
//            <parls>  scS - scaling coefficients for experiment points (2*n). scS(1,:) - mean, scS(2,:) - st. dev.
//            <parls>  scY - scaling coefficients for response values.
//            <parls>  alfa - parameter for estimation of nugget and variance. Used in case when noise is not known.
//            <parls>  coefcov - parameter, that links R to covariance. cefcov*R=covariance matrix. In case R is covariance matrix coefcov=1.
//            <parls>  estimateNoise - flag that specifies if noise should be estimated.=1 when noise is known, =0 when there is no noise, =-1 when noise is not known and should be estimated.
//            <parls>  estimateBeta - flag that specifies if trend coefficients should be estimated. =1 trend parameters are known and should not be estimated.  =-1 trend parameters are not known and should be estimated (In this case the estimation error is acounted in predicted variance).
//            <parls>  estimateSigma -flag that specifies if process variance should be estimated.=1 variance is known and should not be estimated.=-1 variance is not known and should be estimated.
//            <parls>  noise - noise variance. It is equal to inNoise if given, estimated if inNoise is not given.
//            <parls>  sigma - process total variance. It is equal to inSigma if given, estimated if  inSigma is not given.
//		<parls>  silent - if it is necessary to operate in silent mode (no output to console). 
//	       <parls>  optimizer - mlist that contians information about parameter estimation procedure (optimizer with settings).  It is set in findTheta().
//
// Description 
//               Creates kriging model, based on given parameters. It may estimate some parameters analytically. Usually used togeather with findTheta(.), which estimates model parameters.
//
//
//
//This function and other functions operating with kriging model sets and uses all or some of global variables:
//
//            S_kriging - design of experiments;
//            D_kriging - distances between experimental points;
//            Y_kriging - response values for experiment;
//            R_kriging - correlation (covariance) matrix;
//            
//            T_kriging - Cholesky decomposition of R_kriging;
//            Ty_kriging - Ty_kriging=(T')\(Y_kriging);
//            F_kriging -  Trend matrix (m*p) of experiment points
//            M_kriging -   M_kriging=(T_kriging')\(F_kriging);
//            Beta_kriging - estimated trend coefficients or inBeta if given;
//            Tyfb - Tyfb=(T_kriging')\(Y_kriging-F_kriging*Beta_kriging);
//            frf  -  frf=inv((M')*M);
//
//Examples
//// 1D example
//// zero noise
//ftest=sin; // Response function
//
//x_bound=[-2; 2];
//X=(linspace(x_bound(1),x_bound(2),4))'; // Points of experiment
//YExp=ftest(X);// Response values
//
//noise=0;theta=1
//[kmodel] =km(X,YExp,0,'gaussian',theta,noise);// creates kriging model  (constant trend)
//
//
//lob   = [0.1];upb   = [10];
//[kmodel p]=findTheta(kmodel,lob,upb,'MLL','RS',200);// searches for optimal hyper-parameters
////[kmodel p]=findTheta(kmodel,lob,upb,'CV','RS',200);// searches for optimal hyper-parameters
//
//
//
//Xtest=(linspace(x_bound(1),x_bound(2),100))'; // 100 point test sample
//[YX vY]=predict(kmodel,Xtest); // kriging prediction
//
//// plot 
//scf();
//xtitle('Kriging prediction, data points and test function');
//plot(Xtest,YX,'k+');
//plot(Xtest,ftest(Xtest));
//plot(X,YExp,'r+');
//a=gca(); 
// a.data_bounds=a.data_bounds.*[1.1,1;1.1,1];
// a.tight_limits="on";
// 
//
//
//// Simulate conditional Gaussian
//kmodel.noise=0.00000001;// add small noise for numerical stability
// 
//[Sim1]=SimCondGaussian(Xtest,kmodel,50); 
//scf;
//xtitle('Simulation of Conditional Gaussian processes and Kriging mean');
//plot(Xtest,Sim1,'k'); // simulations
//plot(Xtest,YX,'r');// Kriging mean
//
//showModel(kmodel)
//
//
// //See demos directory for more examples.
//See also
//           findTheta
//           predict
//           EGO
//           simGaussian
//           SimCondGaussian
//
//
//Authors
//          Janis Janusevskis, 3MI/EMSE
//Bibliography 
//
//Rasmussen C.E., Williams C.K.I. (2006), Gaussian Processes for Machine Learning, the MIT Press
//
//Roustant O.,Ginsbourger D., Deville  Y. (2010), Two R packages for the analysis of computer experiments by kriging-based metamodeling and optimization. http://hal.archives-ouvertes.fr/hal-00495766_v2/
//
//
//
global R_kriging  Y_kriging T_kriging D_kriging S_kriging F_kriging Beta_kriging;  // largest matrices are global, no passing by value to functions necessary. could be problematic in case of several kriging models

 Y_kriging=Resp;
 S_kriging=Exp;


if ~exists('inNoise','local') then estimateNoise=-1; noise=%nan; elseif inNoise==0 then estimateNoise=0; noise=%eps*10;  else estimateNoise=1;noise=inNoise; end;// known nugget.
if ~exists('inBeta','local')  then estimateBeta=-1; Beta_kriging=%nan;  else estimateBeta=1; Beta_kriging=inBeta;   end;// known Beta.
if ~exists('inSigma','local') then estimateSigma=-1;sigma=%nan; else estimateSigma=1;sigma=inSigma; end;// known Sigma.

// other tests on input for dimensionality and positiveness for noise and sigma
if exists('inBeta','local') then [mt,nt]=size(inBeta); if nt~=1 then error('Error: inBeta  should be column vector of dimensions p*1.');end;end;

if exists('inNoise','local') then [mt,nt]=size(inNoise); if inNoise <0 | mt~=1 | nt~=1 then error('Error: Noise variance must be non negative scalar.');end;end;

if exists('inSigma','local') then [mt,nt]=size(inSigma); if inSigma <0 | mt~=1 | nt~=1 then error('Error: Process variance should be non negative scalar.');end;end;

if ~exists('silent','local') then silent=%f; end;


if estimateNoise==1 & estimateSigma==-1 then
if  ~silent then disp('Warrning: Model can not estimate Sigma analiticaly. Please input it as parameter inSigma or call function findTheta to estimate it numerically. Assumes Sigma=variance(Y)-inNoise.');end;
  sigma=variance(Y_kriging)-noise;
end; 

alfa=%nan;

if estimateNoise==-1  then
  alfa=1-%eps*100; 
  if ~silent then disp('Warrning: Model can not estimate Sigma and Noise analiticaly. Assumes Zerro noise. Use function findTheta to estimate noise numerically'); end;
end;
  

[m,n] = size(S_kriging);
[my,ny]=size(Y_kriging);
if my <> m then error('Number of response values should be equal to the number of experiments.'); end;

if ~exists('trend','local') then trend=poly0;
   elseif typeof(trend)~='fptr' then
        if trend==0 then trend=poly0;
        elseif trend==1 then trend=poly1;
        elseif trend==2 then trend=poly2;
		elseif trend==3 then trend=poly3;
        end;
  elseif typeof(trend)~='function' & typeof(trend)~='fptr' then error('trend should be pointer to function.');
  end;

[mt,nt]=size(theta);
if nt~=n | mt~=1 then error('theta should be row vector of dimensions 1*n.');end;
if or(theta <=0) then error('Values of theta should be > 0.');end; // maybe not needed for some correlation functions??
 

// Normalize Data
// Maybe normalize to [-1,1]
mS = mean(S_kriging,'r');   sS = stdev(S_kriging,'r');
mY = mean(Y_kriging,'r');   sY = stdev(Y_kriging,'r');

// no normalization for now! if change check for scaling in simcongaussian.sci and elsewere.
mS=0;//zeros(1,n);
sS=1;//ones(1,n);
mY=0; sY=1;

j = find(sS == 0);
if  ~isempty(j) then sS(j) = 1; end
j = find(sY == 0);
if  ~isempty(j) then sY(j) = 1; end

S_kriging = (S_kriging - (ones(m,n) .*. mS)) ./ (ones(m,n) .*. sS);
Y_kriging = (Y_kriging - (ones(m,1) .*. mY)) ./ (ones(m,1) .*. sY);


  // calculates distance between points
  mzmax = m*(m-1) / 2;        // number of distances 
  D_kriging = zeros(mzmax,n);        // initialize matrix with distances

  ll=0;
  for k = 1:m-1
    for j=k+1:m
      ll=ll+1;
      D_kriging(ll,:)  = (S_kriging(k,:)) - S_kriging(j,:); // differences between points
    end;
end;


model = mlist(['kmodel','m','n','trend','cov','theta','MLL','dMLL','scS','scY','alfa','coefcov','estimateNoise','estimateBeta','estimateSigma','noise','sigma','silent','optimizer'],          m,n,trend,fcov,theta,%nan,%nan,[mS;sS],[mY;sY],alfa,%nan,estimateNoise,estimateBeta,estimateSigma,noise,sigma,silent,%nan);

kmodel = model;

//[R_kriging,dR]=CovMatrix(m,D,cov,nugget,theta);
model=CorrMatrix(model);// crates correlation/covariance matrix R_kriging 

F_kriging=trend(S_kriging); //creates trend matrix at design points 

decompose_estBeta(model);// calculates T_kriging,estimates Beta etc.

[model]=EstimateVar(model); // estimates variance

//[MLL,dMLL]=logLL(Y_kriging,T_kriging,Ty_kriging);
[model]=logLL(model); // estimates Log-Likelihood value for given parameters.


kmodel = model; 
if ~model.silent then   printf("Creation of kriging model succeeded.\n"); end;
endfunction


